//////////////////////////////////////////////////////////////////////////////////
// Version 1.3
// Note:
//  USE THIS AT YOUR OWN RISK.  YOU SHOULD SAVE THE TARGET DIRECTORY BEFORE
//  EXECUTING THIS PROGRAM.  A FULL SYSTEM SAVE IS ADVISED.
//
//////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////
// CleanNames - Clean up invalid directory and file names such as:
//					 "*.*", "*name", "\name", name with embedded null, name with a
//              unicode character, etc.  Does not work for names of ".", "..",
//              or names with a forward slash ('/').
//
// Command syntax:
//    CleanNames SystemName TargetDir [option]
//
//			SystemName 	- AS/400 system name as entered in DNS or local hosts table
//								or the AS/400 system IP address. The name "localhost"
//								is a valid name when run from the AS/400 jvm
//			TargetDir	- The directory from which to start work.  This is the
//								directory to delete all files from or the directory
//								to search for other directories with invalid names.
//			[option]		- The menu option to execute.  If not supplied the user
//								is prompted for one of following options:
//   
//						  		1	Rename all files in the target directory to a valid name.
//									The names are qfrecov1, qfrecov2, etc...
//								2	Rename all directories in the target directory to a valid name.
//									The names are qdrecov1, qdrecov2, etc..."
//								3  List all objects in the target directory.  Prompt
//									to rename.
//
//
// Other Alternatives
// ==================
// 1) FTP 
//    FTP can be used to rename and delete files and directories with invalid
//    names but standard ANSI characters.  It can remove names with a backslash ('\').
//    It cannot handle names with embedded nulls or unicode characters.
//    Commands:  rename and delete for files, RMDIR for directories (the directory
//    must be empty).  See support document 15234725.
//
// 2) OpNav File Systems, IFS
//    This can handle some names that Windows considers invalid (and hence
//    network drives can't handle).  This includes names like *.* or *dir and
//    many special ANSI characters such as the trademark symbol (TM) and so on..
//
// 3) WrkLnk/rmvlnk/renlnk/rmvdir
//    The 400 wrklnk is not unicode enabled.  It cannot work with files or
//    directories that have names containing embedded nulls, or names
//		containing ANSI and unicode characters that do not exist in the 
//    AS/400 job CCSID. Typically files and directories can be deleted by using
//    a wildcard (rmvlnk '/test/*' or rmvdir '/test/*').  This technique
//		will not work for nested directories where you have to delete files and
//		directories under the corrupted directory first.  See support document 19491120.
//
// 4) IFS Apis
//    As of R450, some APIs needed for cleanup are not unicode enabled.
//   
//////////////////////////////////////////////////////////////////////////////////


import java.io.*;
import java.util.*;
import com.ibm.as400.access.*;

public class CleanNames extends Object
{
	final String FPREFIX = "qfrecov";	//Files renamed to this value + some index
	final String DPREFIX = "qdrecov";	//Dirs  renamed to this value + some index
	final String OPREFIX = "qorecov";	//Obj   renamed to this value + some index

	//Valid Options
	static final int OPT_ALLFILES = 1;		
	static final int OPT_ALLDIRS = 2;
	static final int OPT_PROMPT = 3;

	private AS400 	as400 		= null;		//Toolbox system object
	private String system 		= null;		//Parms
	private String dirName 		= null;
	private Integer menuOption	= null;
	private int		idx			= 0;

	public static void main(String[] parameters)
	{
		CleanNames theApp = new CleanNames();		//Create ourselves
   	System.out.println( " " );

	 	if	( !theApp.ParseParms(parameters) )					
		{
			theApp.showSyntax();
			System.exit(0);
		}
		
		System.out.println("Executing option " + theApp.menuOption + "...");						

		// Create an AS400 object for the AS/400 system that holds the files.
		theApp.as400 = new AS400(theApp.system);		
		//theApp.as400 = new AS400();		
		try {
			//theApp.as400.setUserId("*current");
			//theApp.as400.setPassword("*current");
		} catch (Exception e) {}

		switch ( theApp.menuOption.intValue() )
		{
			case OPT_ALLFILES: 			//rename all files
				if (! theApp.confirm()) break;
				theApp.renameAll();				
				System.out.println("Processed all files ");
				break;
			case OPT_ALLDIRS: 			//rename all dirs
				if (! theApp.confirm()) break;
				theApp.renameAll();
	     		System.out.println("Processed all directories ");
	     		break;
			case OPT_PROMPT: 			//pick from list
				theApp.renameSelected();
				System.out.println("Ending... ");
				break;
	    	default:
				System.out.println("Invalid option.");
				theApp.showOptions();
		}
			
      System.exit(0);
   }	//end main
	
	////////////////////////////////////////////////////////////
	private boolean confirm()
	{
		try
		{
			System.out.print("WARNING:  All ");
			if (menuOption.intValue() == OPT_ALLFILES)
			 	System.out.print("files ");
			else 
				System.out.print("directories ");
			System.out.print("in " + dirName + " will be renamed.\n");  
			System.out.println("Enter 'y' to continue, anything else to exit:");
			byte option[];		
			option = new byte[1];
			System.in.read(option);
			if ( option[0] == 'y' || option[0] == 'Y' ) 
				return true;
			else
				System.out.println("CleanNames ending...");
		}
	   catch (Exception e)
	   {
	      // If any of the above operations failed say the list failed
	      // and output the exception.
	      System.out.println("Error parsing confirm");
	      System.out.println(e);
	   }
		return false;
	}

	////////////////////////////////////////////////////////////
	private int getObjIdx(int start, int end)
	{
		int objIdx = -1;
		try
		{
			System.out.println("Enter number of file to be renamed, -1 to exit: ");
			byte option[];		
			option = new byte[10];
			int count =  System.in.read(option) ;
			//System.out.println("count:" + count + "option: " + option[0] + " " + option[1] + " " + option[2] + " " + option[3] + " ");
			String s = new String( option, 0, count );
			s.trim();		//not working??
			//System.out.println("Len s: " + s.length() +" s:" + s + ":");
			try {
				s = s.substring( 0,  s.indexOf( "\r") );
			} catch (Exception e ){}
			try {
				s = s.substring( 0,  s.indexOf( "\n") );
			} catch (Exception e ){}
			objIdx = Integer.valueOf( s ).intValue();

			if ( (objIdx < start || objIdx > end) && 		//In range
					objIdx != -1									//Not end code
				)
			{
				System.out.println("Invalid selection. Must be " + start + " to " + end);
				objIdx = -1;		//invalid selection
			}
		}
	   catch (Exception e)
	   {
	    	System.out.println("Invalid selection. Must be " + start + " to " + end);
	      System.out.println(e);
	   }
		return objIdx;
	}

	/////////////////////////////////////////////////////////////////////////////
	// Renames all directories or files in target directory
	//TODO:  pass in user specified filter?
	private boolean renameAll()	
	{
	   try
	   {	
	      IFSFile directory = new IFSFile(as400, dirName);
			//TODO: confirm choice
			////////////////////////////////////////////////////////////////////////
			//List out the files
	      // Generate the list of name.  Pass the list method the
	      // directory filter object and the search match criteria.
	      //
	      // Note - this example does the processing in the filter
	      // object.  An alternative is to process the list after
	      // it is returned from the list method call.
			// Create the IFSFile object for the directory.
			String[] directoryNames = null;
			if (menuOption.intValue() == OPT_ALLDIRS)
		      directoryNames = directory.list(new directoryFilter(),"*");
			else
				directoryNames = directory.list(new fileFilter(),"*");

	      // Tell the user if the directory doesn't exist or is empty
	      if (directoryNames == null)
	         System.out.println("The directory does not exist");	
	      else if (directoryNames.length == 0)
	         System.out.println("The directory is empty");		
			else
			{
				//for (int i = 0; i < directoryNames.length; i++)
				//	renameObj( directoryNames[i], i );				
				//System.out.println("Processed " + i + " names.");
			}
	   }
	   catch (Exception e)
	   {
	      // If any of the above operations failed say the list failed
	      // and output the exception.
	      System.out.println("Unable to create list");
	      System.out.println(e);
	   }

		return true;
	}

	/////////////////////////////////////////////////////////////////////////////
	private boolean renameSelected()	
	{
	   try
	   {	
	      IFSFile directory = new IFSFile(as400, dirName);
			System.out.println("Attempting to process " + dirName);

			//List out the files
			String[] directoryNames = null;
	      directoryNames = directory.list();

	      // Tell the user if the directory doesn't exist or is empty
	      if (directoryNames == null)
	         System.out.println("The directory does not exist");	
	      else if (directoryNames.length == 0)
	         System.out.println("The directory is empty");		
			else
			{
				int rsp = -1;
				IFSFile file = null;
				int i = 0;
				do 
				{							
					directoryNames = directory.list();
					//list the objects
					for (i = 0; i < directoryNames.length; i++)
					{
						file = new IFSFile(as400, directoryNames[i]);
					   System.out.print( i + "  " );				//Index
				   	// Print the name of the current file
				      System.out.print( file.getName() );				
				      // Pad the output so the columns line up
				      for (int j = file.getName().length(); j < 18; j++)
					      System.out.print(" ");				
				      // Print the date the file was last changed.
				      System.out.print( new Date(file.lastModified()) );
				      System.out.print("  ");
						if (file.isDirectory())
				         System.out.println("<DIR>");
						else
							System.out.println(file.length());
					}
					//prompt for the object to rename
					rsp = getObjIdx(0, i--);
										
					if (rsp != -1)	
						renObj( dirName, directoryNames[rsp], OPREFIX + idx++ );
				} while (rsp != -1);
			}
	   }
	   catch (Exception e)
	   {
	      // If any of the above operations failed say the list failed
	      // and output the exception.
	      System.out.println("Unable to create list");
	      System.out.println(e);
	   }

		return true;
	}

	/////////////////////////////////////////////////////////////////////////////
	private void renObj (String dir, String oldName, String newName) 
																//throws AS400SecurityException 
																	//,IOException 
	{
		try 
		{			
	      System.out.println("Attempting to rename " + dir + "/" + oldName +
										" to " + newName);
			IFSFile file = new IFSFile(as400, dir, oldName); 
	     	if (! file.exists() )
	      	System.out.println ("\tFile " + file.getAbsolutePath() + " not found");
			else	//rename it
			{
				IFSFile f = new IFSFile(as400, file.getParent(), newName );			
			 	if ( file.renameTo( f ) )
					System.out.println("\tNew name: " + file.getAbsolutePath());
				else
					System.out.println("\tRename rc: false (not always accurate)");
			}
		}
		catch (Exception e)
		{
			System.out.println("\tRename failed" );
			System.out.println("\t" + e);
		} //end catch

		return;		
	}

	/////////////////////////////////////////////////////////////////////////////
	private void renObj (IFSFile file, String newName) 
																//throws AS400SecurityException 
																	//,IOException 
	{
		try 
		{			
	      System.out.println("Attempting to rename " + file.getAbsolutePath() +
										" to " + newName);
	     	if (! file.exists() )
	      	System.out.println ("\tFile " + file.getAbsolutePath() + " not found");
			else	//rename it
			{
				IFSFile f = new IFSFile(as400, file.getParent(), newName );			
			 	if ( file.renameTo( f ) )
					System.out.println("\tNew name: " + file.getAbsolutePath());
				else
					System.out.println("\tRename rc: false (not always accurate)");
			}
		}
		catch (Exception e)
		{
			System.out.println("\tRename failed" );
			System.out.println("\t" + e);
		} //end catch

		return;		
	}


	/////////////////////////////////////////////////////////////////////////////
	private boolean ParseParms(String[] parms)
	{

      if ( parms.length == 2 || parms.length == 3 )
		{
			try
			{
				system = parms[0];
   	      dirName = parms[1];
				if ( parms.length == 3 )
					//int i = Integer.parseInt(  );
					menuOption = Integer.valueOf( parms[2] );
				else
				{
					showOptions();
					byte option[];		
					option = new byte[1];
					System.out.println("Enter option (1, 2 or 3): " );					
					System.in.read(option);
					menuOption = Integer.valueOf( new String( option  ) );
				}
			}
			catch (Exception e)
			{
				System.out.println("Error parsing program parameters");
				System.out.println("\t" + e );
				return false;
			}
		}		
		else		//incorrect number of params
		{	
			System.out.println("Incorrect number of program parameters.");
			return false;
		}
		return true;
	}

	private void showSyntax()
	{
         System.out.println("");
         System.out.println("");
         System.out.println("CleanNames SystemName TargetDir [option]");
         System.out.println("");
         System.out.println("\tWhere");
         System.out.println("");
			System.out.println("\tSystemName - AS/400 system name as entered in DNS or local hosts");
			System.out.println("\t\ttable. Can also be the AS/400 system IP address. From the ");
			System.out.println("\t\tAS/400 jvm use the special value \"localhost\".");
			System.out.println("");
			System.out.println("\tTargetDir	- The directory from which to start work.  This is");
			System.out.println("\t\tthe directory to delete all files from or the directory");
			System.out.println("\t\tto search for other directories with invalid names.");
			System.out.println("\t\tThe directory name should start with a forward slash ('/').");
			System.out.println("");
			System.out.println("\t[option] - The menu option to execute.  If not supplied the user");
			System.out.println("\t\tis prompted.");
			System.out.println("");

			showOptions();

         System.out.println("");
         System.out.println("For example:");
         System.out.println("");
         System.out.println("   c:\\>java CleanNames LOCALHOST /dir1/dir2 1 >> /CleanNames.log");
         System.out.println("");
         System.out.println("");

		return;
	}

	private void showOptions()
	{
      System.out.println("Option:");
      System.out.println("\t" + OPT_ALLFILES + " Rename all files in the target directory to a valid name.");
      System.out.println("\t\tThe names are qfrecov1, qfrecov2, etc...");
      System.out.println("\t" + OPT_ALLDIRS + " Rename all directories in the target directory to a valid name.");
      System.out.println("\t\tThe names are qdrecov1, qdrecov2, etc...");
      System.out.println("\t" + OPT_PROMPT + " List all objects in the target directory.  Prompt ");
      System.out.println("\t\tto rename. ");
		return;
	}

	private void crtDir (String path, String dir) throws IOException, AS400SecurityException
	{
		IFSFile aDir =new IFSFile(as400, path + "/" + dir);
		System.out.println("Attempting directory: " + path + "/" + dir);

		//Delete it if it exists
		if (aDir.delete() == false)
			System.err.println("\tUnable to delete existing dir.");
		else
			System.err.println("\tDeleted existing dir.");

		//Create the directory.
	  	if ( aDir.mkdir() )
			System.out.println("\tCreated directory " + aDir.getAbsolutePath());
		else
			System.out.println("\tFailed Creating directory " + aDir.getAbsolutePath());

		return;
	}

	////////////////////////////////////////////////////////////////////////////////////
	// Creates a new file and adds a few bytes of data
	// NOTE:  Deletes any existing file!
	private void crtF (String dir, String fName) throws IOException, 
																		AS400SecurityException
	{
		//This will not create the file. Create a file object that represents the file.				
		//aFile =new IFSFile(as400, directoryName + "/myfile.txt" );

		java.io.File file       = null;
     	java.io.OutputStream os = null;
		
      System.out.println("Attempting to create " + dir + "/" + fName);
		file = new IFSJavaFile(as400, dir, fName); 
     	if (file.exists())
      	System.out.println ("\tCreated IFSJavaFile " );      
     	else
      	System.out.println ("File " + file.getName() + " not found");

		// Delete any existing file. 
		if (file.delete() == false)
			System.err.println("\tUnable to delete existing file.");
		else
			System.err.println("\tDeleted existing file.");

		os = (OutputStream)new IFSFileOutputStream(as400, (IFSJavaFile)file, 
																IFSFileOutputStream.SHARE_NONE, false);
		writeData(file, os);	//must write to it to create it.
		os.close();	

     	System.out.println("\tCreated " + file.getAbsolutePath() + " length: " +
			   				file.length() + " Modified: " + 
								new Date(file.lastModified()) );
		return;		
	}

	private	void writeData (java.io.File file, java.io.OutputStream os)
								   throws IOException
	{
		// Determine the parent directory of the file.
		System.out.println ("\tDirectory: " + file.getPath());	
		// Determine the name of the file.
		System.out.println ("\tName: " + file.getName());	
		// Determine when the file was last modified.
		//System.out.println ("\tDate: " + new Date(file.lastModified()));
		
		//System.out.println ("Writing Data");
		for (int i = 0; i < 256; i++)
		  os.write ((byte)i);

		return;
	}
		 
}	//end CleanNames Class



////////////////////////////////////////////////////////////////////////////
//
// The directory filter class prints information from the file object.
//
// Another way to use the filter is to simply return true or false
// based on information in the file object.  This lets the mainline
// function decide what to do with the list of files that meet the
// search criteria.
//
////////////////////////////////////////////////////////////////////////////

class directoryFilter implements IFSFileFilter
{
	private int idx = 0;
	final String FPREFIX = "qfrecov";
	final String DPREFIX = "qdrecov";

   public boolean accept(IFSFile file)
   {
      try
      {
         if (file.isDirectory())
			{
            //System.out.println("<DIR>");
				try 
		      {
			   	// Print the name of the current file
			      System.out.print( file.getName() );
			
			      // Pad the output so the columns line up
			      for (int i = file.getName().length(); i < 18; i++)
				      System.out.print(" ");
			
			      // Print the date the file was last changed.
			      System.out.print( new Date(file.lastModified()) );
			      System.out.print("  ");
			
			      // Print if the entry is a file or directory
		         System.out.println("<DIR>");
					//	         System.out.println(file.length());
					IFSFile f = new IFSFile(file.getSystem(), file.getParent(),
									 	DPREFIX + idx++ );
					//System.out.println("\tAttempt rename: " + f.getAbsolutePath() );
				 	if ( file.renameTo(f) )						
						System.out.println("\tNew name: " + file.getAbsolutePath());
					else
						System.out.println("\tRename failed.");
				}
			   catch (Exception e)
			   {
			      // If any of the above operations failed say the list failed
			      // and output the exception.
			      System.out.println("\tUnable to rename object");
			      System.out.println(e);
			   }
				return true;
			}
         else
				return false;
		}
		catch (Exception e)
    	{
			System.out.println("Unexpected error in directoryFilter");
			System.out.println("\t" + e);
			System.out.println("");
         return false;
      }
   }	//end accept
}	//end class directoryFilter

class fileFilter implements IFSFileFilter
{
	private int idx = 0;
	static final String FPREFIX = "qfrecov";
	static final String DPREFIX = "qdrecov";

   public boolean accept(IFSFile file)
   {
      try
      {
         if (file.isDirectory())
            //System.out.println("<DIR>");
				return false;
         else
			{
				try 
		      {
			   	// Print the name of the current file
			      System.out.print( file.getName() );
			
			      // Pad the output so the columns line up
			      for (int i = file.getName().length(); i < 18; i++)
				      System.out.print(" ");
			
			      // Print the date the file was last changed.
			      System.out.print( new Date(file.lastModified()) );
			      System.out.print("  ");
			
			      // Print if the entry is a file or directory
		         //System.out.println("<DIR>");
					System.out.println(file.length());
					IFSFile f = new IFSFile(file.getSystem(), file.getParent(),
									 	FPREFIX + idx++ );
					//System.out.println("\tAttempt rename: " + f.getAbsolutePath() );
				 	if ( file.renameTo(f) )						
						System.out.println("\tNew name: " + file.getAbsolutePath());
					else
						System.out.println("\tRename failed.");
				}
			   catch (Exception e)
			   {
			      // If any of the above operations failed say the list failed
			      // and output the exception.
			      System.out.println("\tUnable to rename object");
			      System.out.println(e);
					// display any file at exit return false;
			   }
	
				return true;
			}
		}
		catch (Exception e)
    	{
			System.out.println("Unexpected error in fileFilter");
			System.out.println("\t" + e);
			System.out.println("");
         return false;
      }
   }	//end accept
}	//end class fileFilter





